from typing import Optional, Union

from pydantic import AnyUrl, BaseModel, EmailStr, Extra, validator


class Contact(BaseModel):
    """Contact information for the exposed API."""

    name: Optional[str] = None
    """
    The identifying name of the contact person/organization.
    """

    url: Optional[AnyUrl] = None
    """
    The URL pointing to the contact information.
    MUST be in the form of a URL.
    """

    email: Optional[Union[EmailStr, str]] = None
    """
    The email address of the contact person/organization.
    MUST be in the form of an email address.
    """

    @validator("email", pre=True)
    def validate_email(  # pylint: disable=no-self-argument
        cls,
        v: Union[EmailStr, str],
    ) -> EmailStr:
        """Validates that email is a valid email address.

        Args:
            v: Holds the email string to be validated

        Raises:
            ValueError: Value is not a valid email address

        Returns:
            Validated email string.
        """
        if isinstance(v, str):
            v = EmailStr(v)
        return v

    class Config:
        extra = Extra.ignore
        schema_extra = {
            "examples": [
                {
                    "name": "API Support",
                    "url": "http://www.example.com/support",
                    "email": "support@example.com",
                }
            ]
        }



from pydantic import Extra, Field
from typing_extensions import Literal

from .parameter import Parameter


class Header(Parameter):
    """The Header Object follows the structure of the [Parameter
    Object](https://spec.openapis.org/oas/v3.1.0#parameterObject) with the
    following changes:

    1. `name` MUST NOT be specified, it is given in the corresponding `headers` map.
    2. `in` MUST NOT be specified, it is implicitly in `header`.
    3. All traits that are affected by the location MUST be applicable to a location of `header`
       (for example, [style](https://spec.openapis.org/oas/v3.1.0#parameterStyle)).
    """

    name: Literal[""] = Field(default="", const=True)
    param_in: Literal["header"] = Field(default="header", const=True, alias="in")

    class Config:
        extra = Extra.ignore
        allow_population_by_field_name = True
        schema_extra = {
            "examples": [
                {
                    "description": "The number of allowed requests in the current period",
                    "schema": {"type": "integer"},
                }
            ]
        }
